#include "bzfsAPI.h"
#include "generic.h"

//Structure is used for defining properties of both teams.
typedef struct {
	bz_eTeamType color; //Number of team colors.
	int vip; //The player slot that is the current VIP.
	unsigned int lives; //The number of lives the team has.
	bool vipnotify; //Whether or not the VIP of that team has been notified they are VIP yet.
	int vipselected; //The requested player slot that will become the next VIP.
	int score; //The team score.
} team;

class vip : public bz_Plugin, bz_CustomSlashCommandHandler
{
	virtual const char* Name (){return "VIP";}
	virtual void Init ( const char* config);
	
	virtual bool SlashCommand (int playerID,bz_ApiString command,bz_ApiString message,bz_APIStringList* params);

	virtual void Event ( bz_EventData * eventData );
	
	public:
		team team1,team2;
		bool running; //Whether or not the game is currently running.
		int lives;
		double nextspawntime; //The server time has to be passed this value to allow players to spawn.
		int delay;
};

BZ_PLUGIN(vip)


//Custom Helper Functions

void NotifyVIP(int playerID,int VIPID,int lives) {
	bz_sendTextMessagef(BZ_SERVER,playerID,"%s is the current VIP and has %d li%s remaining. Defend your team's VIP and find the opponent's!",bz_getPlayerCallsign(VIPID),lives,(lives!=1)?"ves":"fe");
}
void NotifyVIP(bz_eTeamType playerID,int VIPID,int lives) {
	bz_sendTextMessagef(BZ_SERVER,playerID,"%s is the current VIP and has %d li%s remaining. Defend your team's VIP and find the opponent's!",bz_getPlayerCallsign(VIPID),lives,(lives!=1)?"ves":"fe");
}
void DamagedVIP(int playerID,int VIPID,int lives) {
	bz_sendTextMessagef(BZ_SERVER,playerID,"Your team's VIP, %s, has been hit and has %d li%s remaining!",bz_getPlayerCallsign(VIPID),lives,(lives!=1)?"ves":"fe");
}
void DamagedVIP(bz_eTeamType playerID,int VIPID,int lives) {
	bz_sendTextMessagef(BZ_SERVER,playerID,"Your team's VIP, %s, has been hit and has %d li%s remaining!",bz_getPlayerCallsign(VIPID),lives,(lives!=1)?"ves":"fe");
}

void WinText(int playerID) {
	bz_sendTextMessage(BZ_SERVER,playerID," ");
	bz_sendTextMessage(BZ_SERVER,playerID,"__  ______  __  __   _       _______   ____");
	bz_sendTextMessage(BZ_SERVER,playerID,"\\ \\/ / __ \\/ / / /  | |     / /  _/ | / / /");
	bz_sendTextMessage(BZ_SERVER,playerID," \\  / / / / / / /   | | /| / // //  |/ / / ");
	bz_sendTextMessage(BZ_SERVER,playerID," / / /_/ / /_/ /    | |/ |/ // // /|  /_/  ");
	bz_sendTextMessage(BZ_SERVER,playerID,"/_/\\____/\\____/     |__/|__/___/_/ |_(_)   ");
	bz_sendTextMessage(BZ_SERVER,playerID," ");
}
void WinText(bz_eTeamType playerID) {
	bz_sendTextMessage(BZ_SERVER,playerID," ");
	bz_sendTextMessage(BZ_SERVER,playerID,"__  ______  __  __   _       _______   ____");
	bz_sendTextMessage(BZ_SERVER,playerID,"\\ \\/ / __ \\/ / / /  | |     / /  _/ | / / /");
	bz_sendTextMessage(BZ_SERVER,playerID," \\  / / / / / / /   | | /| / // //  |/ / / ");
	bz_sendTextMessage(BZ_SERVER,playerID," / / /_/ / /_/ /    | |/ |/ // // /|  /_/  ");
	bz_sendTextMessage(BZ_SERVER,playerID,"/_/\\____/\\____/     |__/|__/___/_/ |_(_)   ");
	bz_sendTextMessage(BZ_SERVER,playerID," ");
}
void LoseText(int playerID) {
	bz_sendTextMessage(BZ_SERVER,playerID," ");
	bz_sendTextMessage(BZ_SERVER,playerID,"    ______      _ __    ");
	bz_sendTextMessage(BZ_SERVER,playerID,"   / ____/___ _(_) /    ");
	bz_sendTextMessage(BZ_SERVER,playerID,"  / /_  / __ `/ / /     ");
	bz_sendTextMessage(BZ_SERVER,playerID," / __/ / /_/ / / /_ _ _ ");
	bz_sendTextMessage(BZ_SERVER,playerID,"/_/    \\__,_/_/_/(_|_|_)");
	bz_sendTextMessage(BZ_SERVER,playerID," ");
}
void LoseText(bz_eTeamType playerID) {
	bz_sendTextMessage(BZ_SERVER,playerID," ");
	bz_sendTextMessage(BZ_SERVER,playerID,"    ______      _ __    ");
	bz_sendTextMessage(BZ_SERVER,playerID,"   / ____/___ _(_) /    ");
	bz_sendTextMessage(BZ_SERVER,playerID,"  / /_  / __ `/ / /     ");
	bz_sendTextMessage(BZ_SERVER,playerID," / __/ / /_/ / / /_ _ _ ");
	bz_sendTextMessage(BZ_SERVER,playerID,"/_/    \\__,_/_/_/(_|_|_)");
	bz_sendTextMessage(BZ_SERVER,playerID," ");
}

//Slash Command Handling.

bool vip::SlashCommand(int playerID,bz_ApiString command,bz_ApiString message,bz_APIStringList* params)
{
	if (!strcmp("VIP",command.c_str()) or !strcmp("vip",command.c_str())) {
		//This is a VIP command. Accept lowercase also.
		//Verify we have permissions to use these commands.
		if (/*1==1 or <--ONLY FOR TESTING*/ bz_hasPerm(playerID,"VIP") or bz_hasPerm(playerID,"vip")) {
			if (!strcmp("",params->get(0).c_str())) {
				bz_sendTextMessage(BZ_SERVER,playerID,"These commands are available. Type one to see specific syntax:");
				bz_sendTextMessage(BZ_SERVER,playerID,"    /VIP lives    Sets the number of lives the next match will have.");
				bz_sendTextMessage(BZ_SERVER,playerID,"    /VIP setVIP X    Sets the next VIP to a certain teammate.");
				bz_sendTextMessage(BZ_SERVER,playerID,"    /VIP reset    Resets the match.");
			} else
			if (!strcmp("lives",params->get(0).c_str())) {
				//This will adjust the lives.
				if (!strcmp("",params->get(1).c_str())) {
					bz_sendTextMessage(BZ_SERVER,playerID,"Syntax: /VIP lives X");
					bz_sendTextMessage(BZ_SERVER,playerID,"    X is the number of lives.");
				} else {
					int newvalue=atoi(params->get(1).c_str());
					if (newvalue>0) {
						lives=newvalue;
						bz_sendTextMessagef(BZ_SERVER,BZ_ALLUSERS,"The lives have been set to %d for both teams. It will take affect in the next match.",lives);
					} else {
						bz_sendTextMessage(BZ_SERVER,BZ_ALLUSERS,"You must enter a value greater than 0 lives.");
					}
				}
			} else
			if (!strcmp("setVIP",params->get(0).c_str()) or !strcmp("setvip",params->get(0).c_str())) {
				//This will find a player and then make them the next possible VIP.
				if (!strcmp("",params->get(1).c_str())) {
					bz_sendTextMessage(BZ_SERVER,playerID,"Syntax: /VIP setVIP X");
					bz_sendTextMessage(BZ_SERVER,playerID,"    X is the name of a player. No quotes are necessary, even for players with multiple spaces.");
				} else {
					char playername[256];
					int param=1;
					while (strcmp("",params->get(param).c_str())) {
						if (param==1) {
							//First one, no spaces.
							sprintf(playername,"%s",params->get(param).c_str());
						} else {
							sprintf(playername,"%s %s",playername,params->get(param).c_str());
						}
						param+=1;
					}
					int searchplayer=playerNameExists(playername);
					if (searchplayer==-1) {
						//Did not find this user.
						bz_sendTextMessagef(BZ_SERVER,playerID,"Could not find user: %s (Note that it is case sensitive)",playername);
					} else {
						if (bz_getPlayerTeam(playerID)==team1.color and bz_getPlayerTeam(playerID)==bz_getPlayerTeam(searchplayer)) {
							bz_sendTextMessagef(BZ_SERVER,team1.color,"%s set the next VIP for your team to player %s.",bz_getPlayerCallsign(playerID),playername);
							team1.vipselected=searchplayer;
						} else {
							if (bz_getPlayerTeam(playerID)==bz_getPlayerTeam(searchplayer)) {
								bz_sendTextMessagef(BZ_SERVER,team2.color,"%s set the next VIP for your team to player %s.",bz_getPlayerCallsign(playerID),playername);
								team2.vipselected=searchplayer;
							} else {
								bz_sendTextMessagef(BZ_SERVER,playerID,"%s is not on your team!",playername);
							}
						}
					}
				}
			} else
			if (!strcmp("reset",params->get(0).c_str())) {
				bz_sendTextMessagef(BZ_SERVER,BZ_ALLUSERS,"%s has decided to reset the match.",bz_getPlayerCallsign(playerID));
				team1.lives=team2.lives=lives;
				bz_killPlayers();
				bz_resetScore();
				bz_resetTeamScore(team1.color);
				bz_resetTeamScore(team2.color);
				team1.vip=bz_randomPlayer(team1.color);
				team2.vip=bz_randomPlayer(team2.color);
				team1.vipnotify=team2.vipnotify=false;
			}
		} else {
			bz_sendTextMessage(BZ_SERVER,playerID,"You do not have the correct privileges to use the VIP command.");
		}
		return true;
	}
}


//Main Plugin


void vip::Init ( const char* /*commandLine*/ )
{
	bz_debugMessage(2,"VIP plugin loaded");
	Register(bz_ePlayerDieEvent);
	Register(bz_ePlayerPartEvent);
	Register(bz_ePlayerJoinEvent);
	Register(bz_eAllowSpawn);
	Register(bz_eTickEvent);
	
	bz_registerCustomSlashCommand("VIP",this);
	bz_registerCustomSlashCommand("vip",this);
	
	lives=10; //Default to 10 lives.
	delay=0; //Meh, this isn't doing what I expected. We'll let matches start immediately.
	
	nextspawntime=0; //When the server starts, anyone's allowed to join in whenever.
	
	//Make sure we only have two colored teams defined.
	unsigned char definedteams;
	if ((definedteams=(unsigned char)(
		((bool)bz_getTeamPlayerLimit(eRedTeam)?1:0)+
		((bool)bz_getTeamPlayerLimit(eBlueTeam)?1:0)+
		((bool)bz_getTeamPlayerLimit(eGreenTeam)?1:0)+
		((bool)bz_getTeamPlayerLimit(ePurpleTeam)?1:0)
		))!=2
	) {
		if (definedteams>2) {
			bz_debugMessage(0,"ERROR: More than two colored teams have been defined with the VIP Plugin loaded.");
		} else
		if (definedteams<2) {
			bz_debugMessage(0,"ERROR: Less than two colored teams have been defined with the VIP Plugin loaded.");
		}
		bz_shutdown();
	}

	team1.color=team2.color=eNoTeam;
	
	if (bz_getTeamPlayerLimit(eRedTeam)>0) {
		team1.color=eRedTeam;
	}
	if (bz_getTeamPlayerLimit(eGreenTeam)>0) {
		if (team1.color==eNoTeam) {
			team1.color=eGreenTeam;
		} else {
			team2.color=eGreenTeam;
			goto define;
		}
	}
	if (bz_getTeamPlayerLimit(eBlueTeam)>0) {
		if (team1.color==eNoTeam) {
			team1.color=eBlueTeam;
		} else {
			team2.color=eBlueTeam;
			goto define;
		}
	}
	team2.color=ePurpleTeam;
	goto define;
	define:
		team1.lives=team2.lives=lives;
		team1.score=team2.score=0;
		team1.vipnotify=team2.vipnotify=false;
		team1.vip=team2.vip=team1.vipselected=team2.vipselected=-1;
}

void vip::Event(bz_EventData *eventData) {
	switch (eventData->eventType) {
		case bz_ePlayerJoinEvent: {
			int joinedplayer=((bz_PlayerJoinPartEventData_V1*)eventData)->playerID;
			if (bz_getPlayerTeam(joinedplayer)==team1.color) {
				if (team1.vip==-1) {
					//A VIP has not been picked yet. Become it.
					bz_sendTextMessagef(BZ_SERVER,joinedplayer,"You have become the VIP! You have %d lives remaining.",team1.lives);
					team1.vipnotify=true;
					team1.vip=joinedplayer;
				} else {
					NotifyVIP(joinedplayer,team1.vip,team1.lives);
				}
			} else {
				if (bz_getPlayerTeam(joinedplayer)!=eObservers) {
					if (team2.vip==-1) {
						//A VIP has not been picked yet. Become it.
						bz_sendTextMessagef(BZ_SERVER,joinedplayer,"You have become the VIP! You have %d lives remaining.",team2.lives);
						team2.vipnotify=true;
						team2.vip=joinedplayer;
					} else {
						NotifyVIP(joinedplayer,team2.vip,team2.lives);
					}
				}
			}
		}break;
		case bz_ePlayerPartEvent: {
			int partplayer;
			partplayer=((bz_PlayerJoinPartEventData_V1*)eventData)->playerID;
			bz_debugMessagef(2,"%d players remain on this team.",bz_getTeamCount(bz_getPlayerTeam(partplayer))-1);
			if (bz_getPlayerTeam(partplayer)==team1.color) {
				if (team1.vip==partplayer) {
					//We will have to select a new VIP, if we can...
					if (bz_getTeamCount(bz_getPlayerTeam(partplayer))-1>0) {
						if (playerExists(team1.vipselected) and playerNameExists(bz_getPlayerCallsign(team1.vipselected)) and team1.vipselected!=partplayer) {
							//Our VIP we selected is still available. Pick them.
							team1.vip=team1.vipselected;
							team1.vipselected=-1;
						} else {
							//Our VIP we selected is not here, remove it.
							team1.vipselected=-1;
							int limit=1000;
							while (team1.vip==partplayer) {
								team1.vip=bz_randomPlayer(team1.color);
								limit-=1;
								if (limit==0) {
									team1.vip=-1;
									//We will reset all team's lives to be fair.
									team1.lives=lives;
									team2.lives=lives;
									team1.vipnotify=false;
									team2.vipnotify=false;
									bz_resetScore();
									//Re announce when another player joins.
									break;
								}
							}
						}
						bz_debugMessagef(2,"Defined new VIP: %d,%d,%d",team1.color,team1.vip,team1.lives);
						NotifyVIP(team1.color,team1.vip,team1.lives);
					} else {
						team1.vip=-1;
						//We will reset all team's lives to be fair.
						team1.lives=lives;
						team2.lives=lives;
						team1.vipnotify=false;
						team2.vipnotify=false;
						bz_resetScore();
						//Re announce when another player joins.
					}
				}
			} else {
				if (bz_getPlayerTeam(partplayer)!=eObservers) {
					if (team2.vip==partplayer) {
						//We will have to select a new VIP, if we can...
						if (bz_getTeamCount(bz_getPlayerTeam(partplayer))-1>0) {
							if (playerExists(team2.vipselected) and playerNameExists(bz_getPlayerCallsign(team2.vipselected)) and team2.vipselected!=partplayer) {
								//Our VIP we selected is still available. Pick them.
								team2.vip=team2.vipselected;
								team2.vipselected=-1;
							} else {
								//Our VIP we selected is not here, remove it.
								team2.vipselected=-1;
								int limit=1000;
								while (team2.vip==partplayer) {
									team2.vip=bz_randomPlayer(team2.color);
									limit-=1;
									if (limit==0) {
										team1.vip=-1;
										//We will reset all team's lives to be fair.
										team1.lives=lives;
										team2.lives=lives;
										team1.vipnotify=false;
										team2.vipnotify=false;
										bz_resetScore();
										//Re announce when another player joins.
										break;
									}
								}
							}
							NotifyVIP(team2.color,team2.vip,team2.lives);
						} else {
							team2.vip=-1;
							//We will reset all team's lives to be fair.
							team1.lives=lives;
							team2.lives=lives;
							team1.vipnotify=false;
							team2.vipnotify=false;
							bz_resetScore();
							//Re announce when another player joins.
						}
					}
				}
			}
		}break;
		case bz_ePlayerDieEvent: {
			//We only care about the VIP's death.
			int deadplayer,killer;
			deadplayer=((bz_PlayerDieEventData_V1*)eventData)->playerID;
			killer=((bz_PlayerDieEventData_V1*)eventData)->killerID;
			if (bz_getTeamCount(team1.color)>0) {
				bz_resetTeamScore(team1.color);
				bz_setTeamWins(team1.color,team1.score);
			}
			if (bz_getTeamCount(team2.color)>0) {
				bz_resetTeamScore(team2.color);
				bz_setTeamWins(team2.color,team2.score);
			}
			if (killer!=BZ_SERVER) {
				if (bz_getPlayerTeam(deadplayer)==team1.color) {
					if (team1.vip==deadplayer) {
						team1.lives-=1;
						if ((team1.lives<4 and lives>9) or (team1.lives<2 and lives<10)) {
							bz_playSound(team1.color,-1,"flag_alert");
						}
						if (team1.lives<1) {
							//Team 2 wins!
							//Reset lives, reset scores,set new VIPs, display a huge message banner, team score for winning team goes up by 1.
							team1.lives=team2.lives=lives;
							bz_killPlayers();
							bz_resetScore();
							bz_sendTextMessagef(BZ_SERVER,eObservers,"%s Team Won!!",(team2.color==eRedTeam)?"Red":(team2.color==eGreenTeam)?"Green":"Blue");
							bz_sendTextMessage(BZ_SERVER,BZ_ALLUSERS," ");
							bz_sendTextMessagef(BZ_SERVER,eObservers,"The VIPs were %s on the %s team and %s on the %s team.",bz_getPlayerCallsign(team1.vip),(team1.color==eRedTeam)?"Red":(team1.color==eGreenTeam)?"Green":"Blue",bz_getPlayerCallsign(team2.vip),(team2.color==eGreenTeam)?"Green":(team2.color==eBlueTeam)?"Blue":"Purple");
							bz_sendTextMessage(BZ_SERVER,BZ_ALLUSERS," ");
							if (playerExists(team1.vipselected) and playerNameExists(bz_getPlayerCallsign(team1.vipselected))) {
								//Our VIP we selected is still available. Pick them.
								team1.vip=team1.vipselected;
								team1.vipselected=-1;
							} else {
								//Our VIP we selected is not here, remove it.
								team1.vipselected=-1;
								team1.vip=bz_randomPlayer(team1.color);
							}
							if (playerExists(team2.vipselected) and playerNameExists(bz_getPlayerCallsign(team2.vipselected))) {
								//Our VIP we selected is still available. Pick them.
								team2.vip=team2.vipselected;
								team2.vipselected=-1;
							} else {
								//Our VIP we selected is not here, remove it.
								team2.vipselected=-1;
								team2.vip=bz_randomPlayer(team2.color);
							}
							team1.vipnotify=team2.vipnotify=false;
							team2.score+=1;
							WinText(team2.color);
							bz_playSound(team2.color,-1,"flag_won");
							LoseText(team1.color);
							bz_playSound(team1.color,-1,"flag_lost");
							bz_sendTextMessagef(BZ_SERVER,BZ_ALLUSERS,"Current Team Scores:  %s Team - %d    %s Team - %d",(team1.color==eRedTeam)?"Red":(team1.color==eGreenTeam)?"Green":"Blue",team1.score,(team2.color==eGreenTeam)?"Green":(team2.color==eBlueTeam)?"Blue":"Purple",team2.score);
							bz_sendTextMessage(BZ_SERVER,BZ_ALLUSERS," ");
							nextspawntime=bz_getCurrentTime()+delay;
							bz_sendTextMessagef(BZ_SERVER,BZ_ALLUSERS,"Next match is starting! Are you ready?");
						} else {
							DamagedVIP(team1.color,team1.vip,team1.lives);
						}
					}
				} else {
					if (bz_getPlayerTeam(deadplayer)!=eObservers) {
						if (team2.vip==deadplayer) {
							team2.lives-=1;
							if ((team2.lives<4 and lives>9) or (team2.lives<2 and lives<10)) {
								bz_playSound(team2.color,-1,"flag_alert");
							}
							if (team2.lives<1) {
								//Team 1 wins!
								//Reset lives, reset scores,set new VIPs, display a huge message banner, team score for winning team goes up by 1.
								team1.lives=team2.lives=lives;
								bz_killPlayers();
								bz_resetScore();
								bz_sendTextMessagef(BZ_SERVER,eObservers,"%s Team Won!!",(team1.color==eRedTeam)?"Red":(team1.color==eGreenTeam)?"Green":"Blue");
								bz_sendTextMessage(BZ_SERVER,BZ_ALLUSERS," ");
								bz_sendTextMessagef(BZ_SERVER,eObservers,"The VIPs were %s on the %s team and %s on the %s team.",bz_getPlayerCallsign(team1.vip),(team1.color==eRedTeam)?"Red":(team1.color==eGreenTeam)?"Green":"Blue",bz_getPlayerCallsign(team2.vip),(team2.color==eGreenTeam)?"Green":(team2.color==eBlueTeam)?"Blue":"Purple");
								bz_sendTextMessage(BZ_SERVER,BZ_ALLUSERS," ");
								if (playerExists(team1.vipselected) and playerNameExists(bz_getPlayerCallsign(team1.vipselected))) {
									//Our VIP we selected is still available. Pick them.
									team1.vip=team1.vipselected;
									team1.vipselected=-1;
								} else {
									//Our VIP we selected is not here, remove it.
									team1.vipselected=-1;
									team1.vip=bz_randomPlayer(team1.color);
								}
								if (playerExists(team2.vipselected) and playerNameExists(bz_getPlayerCallsign(team2.vipselected))) {
									//Our VIP we selected is still available. Pick them.
									team2.vip=team2.vipselected;
									team2.vipselected=-1;
								} else {
									//Our VIP we selected is not here, remove it.
									team2.vipselected=-1;
									team2.vip=bz_randomPlayer(team2.color);
								}
								team1.vipnotify=team2.vipnotify=false;
								team1.score+=1;
								WinText(team1.color);
								bz_playSound(team1.color,-1,"flag_won");
								LoseText(team2.color);
								bz_playSound(team2.color,-1,"flag_lost");
								bz_sendTextMessagef(BZ_SERVER,BZ_ALLUSERS,"Current Team Scores:  %s Team - %d    %s Team - %d",(team1.color==eRedTeam)?"Red":(team1.color==eGreenTeam)?"Green":"Blue",team1.score,(team2.color==eGreenTeam)?"Green":(team2.color==eBlueTeam)?"Blue":"Purple",team2.score);
								bz_sendTextMessage(BZ_SERVER,BZ_ALLUSERS," ");
								nextspawntime=bz_getCurrentTime()+delay;
								bz_sendTextMessagef(BZ_SERVER,BZ_ALLUSERS,"Next match is starting! Are you ready?");
							} else {
								DamagedVIP(team2.color,team2.vip,team2.lives);
							}
						}
					}
				}
			}
		}break;
		case bz_eAllowSpawn: {
			//Don't allow spawn until 'delay' seconds after a match.
			int spawningplayer;
			spawningplayer=((bz_AllowSpawnData_V1*)eventData)->playerID;
			//Determine which team they are on and announce VIP if it's not done so by now.
			if (!team1.vipnotify and !team2.vipnotify) {
				bz_resetScore();
			}
			if (bz_getPlayerTeam(spawningplayer)==team1.color) {
				if (!team1.vipnotify) {
					NotifyVIP(team1.color,team1.vip,team1.lives);
					team1.vipnotify=true;
				}
			} else {
				if (bz_getPlayerTeam(spawningplayer)!=eObservers) {
					if (!team2.vipnotify) {
						NotifyVIP(team2.color,team2.vip,team2.lives);
						team2.vipnotify=true;
					}
				}
			}
		}break;
		case bz_eTickEvent:{
			//Reset the team counts often to keep accurate score.
			if (bz_getTeamCount(team1.color)>0) {
				bz_resetTeamScore(team1.color);
				bz_setTeamWins(team1.color,team1.score);
			}
			if (bz_getTeamCount(team2.color)>0) {
				bz_resetTeamScore(team2.color);
				bz_setTeamWins(team2.color,team2.score);
			}
		}break;
	}
}