#include <stdio.h>
#include <stdlib.h>
#include <time.h>

/*
	This file is written by Joshua Sigona (sigonasr2)
	and contains some generic BZFlag functions that
	are used frequently in plugins. They are rather flexible.
*/

int bz_getPlayerCount(bool observers=false) {
	return bz_getTeamCount(eRogueTeam)+
	bz_getTeamCount(eRedTeam)+
	bz_getTeamCount(eGreenTeam)+
	bz_getTeamCount(eBlueTeam)+
	bz_getTeamCount(ePurpleTeam)+
	bz_getTeamCount(eRabbitTeam)+
	bz_getTeamCount(eHunterTeam)+
	(observers?bz_getTeamCount(eObservers):0);
}

bool bz_anyPlayers(bool observers=false) {
	return (bool)(bz_getTeamCount(eRogueTeam)+
	bz_getTeamCount(eRedTeam)+
	bz_getTeamCount(eGreenTeam)+
	bz_getTeamCount(eBlueTeam)+
	bz_getTeamCount(ePurpleTeam)+
	bz_getTeamCount(eRabbitTeam)+
	bz_getTeamCount(eHunterTeam)+
	(observers?bz_getTeamCount(eObservers):0));
}

int bz_randomPlayer(bz_eTeamType team=eNoTeam) {
	//Returns the index of a random player. Team is optional.
	//-1 is returned if no one is found.
	srand(time(NULL));
	if (team==eNoTeam) {
		if (bz_anyPlayers()) {
			bz_debugMessage(4,"There are players.");
			bz_APIIntList*playerlist=bz_getPlayerIndexList();
			int picked;
			picked=(*playerlist)[rand()%playerlist->size()];
			bz_deleteIntList(playerlist);
			return picked;
		} else {
			return -1;
		}
	} else {
		if (bz_getTeamCount(team)>0) {
			int picked=0;
			bz_APIIntList*playerlist=bz_getPlayerIndexList();
			while (true) {
				picked=rand()%playerlist->size();
				if (bz_getPlayerTeam(picked)==team) {
					break;
				}
			}
			bz_deleteIntList(playerlist);
			return picked;
		} else {
			return -1;
		}
	}
}

void bz_killPlayers(bz_eTeamType team=eNoTeam,int player=-1) {
	//Kills everyone if team and player are left unchanged.
	//Kills only one team if team is specified, and kills
	//a certain player when player is not -1.
	if (player!=-1) {
		bz_killPlayer(player,false,BZ_SERVER);
	} else
	if (team!=eNoTeam) {
		bz_APIIntList*playerlist=bz_getPlayerIndexList();
		int i;
		for (i=0;i<playerlist->size();i++) {
			if (bz_getPlayerTeam((*playerlist)[i])==team) {
				bz_killPlayer((*playerlist)[i],false,BZ_SERVER);
			}
		}
		bz_deleteIntList(playerlist);
	} else {
		bz_APIIntList*playerlist=bz_getPlayerIndexList();
		int i;
		for (i=0;i<playerlist->size();i++) {
			bz_killPlayer((*playerlist)[i],false,BZ_SERVER);
		}
		bz_deleteIntList(playerlist);
	}
}

void bz_resetScore(bz_eTeamType team=eNoTeam,int player=-1) {
	//Resets the score for everyone if team and player are left unchanged.
	//Resets a team's score if team is specified, and resets
	//a specific player's score only when player is not -1.
	if (player!=-1) {
		bz_resetPlayerScore(player);
	} else
	if (team!=eNoTeam) {
		bz_APIIntList*playerlist=bz_getPlayerIndexList();
		int i;
		for (i=0;i<playerlist->size();i++) {
			if (bz_getPlayerTeam((*playerlist)[i])==team) {
				bz_resetPlayerScore((*playerlist)[i]);
			}
		}
		bz_deleteIntList(playerlist);
	} else {
		bz_APIIntList*playerlist=bz_getPlayerIndexList();
		int i;
		for (i=0;i<playerlist->size();i++) {
			bz_resetPlayerScore((*playerlist)[i]);
		}
		bz_deleteIntList(playerlist);
	}
}

int playerNameExists(const char*callsign) {
	//Cycles through all players seeing if the callsign exists.
	//Returns a player ID that matches.
	int found=-1;
	bz_APIIntList*playerlist=bz_getPlayerIndexList();
	int i;
	const char*temp;
	for (i=0;i<playerlist->size();i++) {
		temp=bz_getPlayerCallsign((*playerlist)[i]);
		if (!strcmp(callsign,temp)) {
			found=(*playerlist)[i];
			break;
		}
	}
	bz_deleteIntList(playerlist);
	return found;
}

bool playerExists(int id) {
	//Cycles through all players seeing if the ID is found.
	//Returns true or false.
	bz_APIIntList*playerlist=bz_getPlayerIndexList();
	int i;
	for (i=0;i<playerlist->size();i++) {
		if ((*playerlist)[i]==id) {
			bz_deleteIntList(playerlist);
			return true;
		}
	}
	bz_deleteIntList(playerlist);
	return false;
}

void bz_playSound(bz_eTeamType team=eNoTeam,int player=-1,const char*sound="") {
	//Plays a sound to a specific player if specified,
	//then sees if a specific team to play it for is specified,
	//otherwise, plays it for everyone in the game.
	if (player!=-1) {
		bz_sendPlayCustomLocalSound(player,sound);
	} else
	if (team!=eNoTeam) {
		bz_APIIntList*playerlist=bz_getPlayerIndexList();
		int i;
		for (i=0;i<playerlist->size();i++) {
			if (bz_getPlayerTeam((*playerlist)[i])==team) {
				bz_sendPlayCustomLocalSound((*playerlist)[i],sound);
			}
		}
		bz_deleteIntList(playerlist);
	} else
	{
		bz_APIIntList*playerlist=bz_getPlayerIndexList();
		int i;
		for (i=0;i<playerlist->size();i++) {
			bz_sendPlayCustomLocalSound((*playerlist)[i],sound);
		}
		bz_deleteIntList(playerlist);
	}
}